package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.SyntaxError;

class TestSingleMnemonics {

  @Test
  void testSWAP() throws SyntaxError {
    SWAP mno = new SWAP("SWAP");
    assertEquals(0x50, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testBSET() throws SyntaxError {
    BSET mno = new BSET("BSET");
    assertEquals(0x51, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      BSET mno1 = new BSET("BSET akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      BSET mno1 = new BSET("BSET 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      BSET mno1 = new BSET("BSET :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSETB() throws SyntaxError {
    SETB mno = new SETB("SETB");
    assertEquals(0x61, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETB mno1 = new SETB("SETB akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETB mno1 = new SETB("SETB 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETB mno1 = new SETB("SETB :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testCSET() throws SyntaxError {
    CSET mno = new CSET("CSET");
    assertEquals(0x52, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      CSET mno1 = new CSET("CSET akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      CSET mno1 = new CSET("CSET 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      CSET mno1 = new CSET("CSET :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSETC() throws SyntaxError {
    SETC mno = new SETC("SETC");
    assertEquals(0x62, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETC mno1 = new SETC("SETC akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETC mno1 = new SETC("SETC 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETC mno1 = new SETC("SETC :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testDSET() throws SyntaxError {
    DSET mno = new DSET("DSET");
    assertEquals(0x53, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DSET mno1 = new DSET("DSET akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DSET mno1 = new DSET("DSET 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DSET mno1 = new DSET("DSET :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSETD() throws SyntaxError {
    SETD mno = new SETD("SETD");
    assertEquals(0x63, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETD mno1 = new SETD("SETD akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETD mno1 = new SETD("SETD 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETD mno1 = new SETD("SETD :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testESET() throws SyntaxError {
    ESET mno = new ESET("ESET");
    assertEquals(0x5d, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      ESET mno1 = new ESET("ESET akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      ESET mno1 = new ESET("ESET 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      ESET mno1 = new ESET("ESET :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSETE() throws SyntaxError {
    SETE mno = new SETE("SETE");
    assertEquals(0x6D, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETE mno1 = new SETE("SETE akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETE mno1 = new SETE("SETE 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETE mno1 = new SETE("SETE :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testFSET() throws SyntaxError {
    FSET mno = new FSET("FSET");
    assertEquals(0x5e, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      FSET mno1 = new FSET("FSET akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      FSET mno1 = new FSET("FSET 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      FSET mno1 = new FSET("FSET :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSETF() throws SyntaxError {
    SETF mno = new SETF("SETF");
    assertEquals(0x6E, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETF mno1 = new SETF("SETF akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETF mno1 = new SETF("SETF 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SETF mno1 = new SETF("SETF :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testPUSH() throws SyntaxError {
    PUSH mno = new PUSH("PUSH");
    assertEquals(0x5f, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testPOP() throws SyntaxError {
    POP mno = new POP("POP");
    assertEquals(0x6f, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic("SWAP", 0);
    assertEquals(SWAP.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("BSET", 0);
    assertEquals(BSET.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("CSET", 0);
    assertEquals(CSET.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("DSET", 0);
    assertEquals(DSET.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("ESET", 0);
    assertEquals(ESET.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("FSET", 0);
    assertEquals(FSET.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("PUSH", 0);
    assertEquals(PUSH.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SETB", 0);
    assertEquals(SETB.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SETC", 0);
    assertEquals(SETC.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SETD", 0);
    assertEquals(SETD.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SETE", 0);
    assertEquals(SETE.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SETF", 0);
    assertEquals(SETF.class, mnemonic.getClass());
  }
}
